Class: Dash::Commands::Server
Constant Summary
Constants inherited from Base
Base::DOCKER_HEALTH_STATUS_FORMAT, Base::NO_HEALTHCHECK
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
- #app_directory_count ⇒ Object
-
#listeners_on(port) ⇒ Object
Lists TCP listeners on the given port, one line per listener, no header.
- #remove_app_directory ⇒ Object
Methods inherited from Base
#container_id_for, #ensure_docker_installed, #ensure_run_directory, #initialize, #make_directory, #make_directory_for, #read_file, #remove_directory, #remove_file, #run_over_ssh
Constructor Details
This class inherits a constructor from Dash::Commands::Base
Instance Method Details
#app_directory_count ⇒ Object
6 7 8 9 10 |
# File 'lib/dash/commands/server.rb', line 6 def app_directory_count pipe \ [ :ls, config.apps_directory ], [ :wc, "-l" ] end |
#listeners_on(port) ⇒ Object
Lists TCP listeners on the given port, one line per listener, no header.
13 14 15 |
# File 'lib/dash/commands/server.rb', line 13 def listeners_on(port) [ :ss, "-ltnH", :sport, "=", ":#{port}" ] end |
#remove_app_directory ⇒ Object
2 3 4 |
# File 'lib/dash/commands/server.rb', line 2 def remove_app_directory remove_directory config.app_directory end |